@@ -1,6 +1,6 @@
 package Net::SAML2::Protocol::LogoutRequest;
 use Moose;
-our $VERSION = '0.58'; # TRIAL VERSION
+our $VERSION = '0.60'; # TRIAL VERSION
 use MooseX::Types::Common::String qw/ NonEmptySimpleStr /;
 use MooseX::Types::URI qw/ Uri /;
 use Net::SAML2::XML::Util qw/ no_comments /;
@@ -148,7 +148,7 @@ Net::SAML2::Protocol::LogoutRequest - SAML2 LogoutRequest Protocol object
 
 =head1 VERSION
 
-version 0.58
+version 0.60
 
 =head1 SYNOPSIS
 
@@ -159,10 +159,6 @@ version 0.58
     session     => $session,
   );
 
-=head1 NAME
-
-Net::SAML2::Protocol::LogoutRequest - the SAML2 LogoutRequest object
-
 =head1 METHODS
 
 =head2 new( ... )
@@ -175,20 +171,12 @@ Arguments:
 
 =item B<session>
 
-session to log out
+Session to log out
 
 =item B<nameid>
 
 NameID of the user to log out
 
-=item B<nameid_format>
-
-NameIDFormat to specify
-
-=item B<issuer>
-
-SP's identity URI
-
 =item B<destination>
 
 IdP's identity URI this is required for a signed message but likely should be
@@ -196,6 +184,31 @@ sent regardless
 
 =back
 
+The following options alter the output of the NameID element
+
+=over
+
+=item B<nameid_format>
+
+When supplied adds the Format attribute to the NameID
+
+=item B<sp_provided_id>
+
+When supplied adds the SPProvidedID attribute to the NameID
+
+=item B<include_name_qualifier>
+
+Tell the module to include the NameQualifier and SPNameQualifier attributes in
+the NameID. Defaults to false unless the B<nameid_format> equals
+C<urn:oasis:names:tc:SAML:2.0:nameidformat:persistent>
+
+=item B<affiliation_group_id>
+
+When supplied sets the SPNameQualifier attribute. When not supplied, this
+defaults to the issuer.
+
+=back
+
 =head2 new_from_xml( ... )
 
 Create a LogoutRequest object from the given XML.
